@@ -44,11 +44,12 @@
 #include "util.h"
 #include "LogFactory.h"
 #include "Logger.h"
-#include "Base64.h"
+#include "base64.h"
 #include "a2functional.h"
 #include "fmt.h"
 #include "SocketRecvBuffer.h"
 #include "TimeA2.h"
+#include "array_fun.h"
 
 namespace aria2 {
 
@@ -68,6 +69,58 @@ HttpServer::HttpServer
 
 HttpServer::~HttpServer() {}
 
+namespace {
+const char* getStatusString(int status)
+{
+  switch(status) {
+  case 100: return "100 Continue";
+  case 101: return "101 Switching Protocols";
+  case 200: return "200 OK";
+  case 201: return "201 Created";
+  case 202: return "202 Accepted";
+  case 203: return "203 Non-Authoritative Information";
+  case 204: return "204 No Content";
+  case 205: return "205 Reset Content";
+  case 206: return "206 Partial Content";
+  case 300: return "300 Multiple Choices";
+  case 301: return "301 Moved Permanently";
+  case 302: return "302 Found";
+  case 303: return "303 See Other";
+  case 304: return "304 Not Modified";
+  case 305: return "305 Use Proxy";
+    // case 306: return "306 (Unused)";
+  case 307: return "307 Temporary Redirect";
+  case 400: return "400 Bad Request";
+  case 401: return "401 Unauthorized";
+  case 402: return "402 Payment Required";
+  case 403: return "403 Forbidden";
+  case 404: return "404 Not Found";
+  case 405: return "405 Method Not Allowed";
+  case 406: return "406 Not Acceptable";
+  case 407: return "407 Proxy Authentication Required";
+  case 408: return "408 Request Timeout";
+  case 409: return "409 Conflict";
+  case 410: return "410 Gone";
+  case 411: return "411 Length Required";
+  case 412: return "412 Precondition Failed";
+  case 413: return "413 Request Entity Too Large";
+  case 414: return "414 Request-URI Too Long";
+  case 415: return "415 Unsupported Media Type";
+  case 416: return "416 Requested Range Not Satisfiable";
+  case 417: return "417 Expectation Failed";
+    // RFC 2817 defines 426 status code.
+  case 426: return "426 Upgrade Required";
+  case 500: return "500 Internal Server Error";
+  case 501: return "501 Not Implemented";
+  case 502: return "502 Bad Gateway";
+  case 503: return "503 Service Unavailable";
+  case 504: return "504 Gateway Timeout";
+  case 505: return "505 HTTP Version Not Supported";
+  default: return "";
+  }
+}
+} // namespace
+
 SharedHandle<HttpHeader> HttpServer::receiveRequest()
 {
   if(socketRecvBuffer_->bufferEmpty()) {
@@ -89,22 +142,39 @@ SharedHandle<HttpHeader> HttpServer::receiveRequest()
     lastBody_.clear();
     lastBody_.str("");
     lastContentLength_ =
-      lastRequestHeader_->getFirstAsUInt(HttpHeader::CONTENT_LENGTH);
+      lastRequestHeader_->findAsLLInt(HttpHeader::CONTENT_LENGTH);
+    if(lastContentLength_ < 0) {
+      throw DL_ABORT_EX("Content-Length must be positive.");
+    }
     headerProcessor_->clear();
 
-    std::string connection =
-      util::toLower(lastRequestHeader_->getFirst(HttpHeader::CONNECTION));
+    const std::string& connection =
+      lastRequestHeader_->find(HttpHeader::CONNECTION);
     acceptsPersistentConnection_ =
-      connection.find(HttpHeader::CLOSE) == std::string::npos &&
+      util::strifind(connection.begin(),
+                     connection.end(),
+                     HttpHeader::CLOSE.begin(),
+                     HttpHeader::CLOSE.end()) == connection.end() &&
       (lastRequestHeader_->getVersion() == HttpHeader::HTTP_1_1 ||
-       connection.find("keep-alive") != std::string::npos);
+       util::strifind(connection.begin(),
+                      connection.end(),
+                      HttpHeader::KEEP_ALIVE.begin(),
+                      HttpHeader::KEEP_ALIVE.end()) != connection.end());
 
-    std::vector<std::string> acceptEncodings;
-    util::split(lastRequestHeader_->getFirst(HttpHeader::ACCEPT_ENCODING),
-                std::back_inserter(acceptEncodings), A2STR::COMMA_C, true);
-    acceptsGZip_ =
-      std::find(acceptEncodings.begin(), acceptEncodings.end(), "gzip")
-      != acceptEncodings.end();
+    std::vector<Scip> acceptEncodings;
+    const std::string& acceptEnc =
+      lastRequestHeader_->find(HttpHeader::ACCEPT_ENCODING);
+    util::splitIter(acceptEnc.begin(), acceptEnc.end(),
+                    std::back_inserter(acceptEncodings), ',', true);
+    acceptsGZip_ = false;
+    for(std::vector<Scip>::const_iterator i = acceptEncodings.begin(),
+          eoi = acceptEncodings.end(); i != eoi; ++i) {
+      if(util::strieq((*i).first, (*i).second,
+                      HttpHeader::GZIP.begin(), HttpHeader::GZIP.end())) {
+        acceptsGZip_ = true;
+        break;
+      }
+    }
     return header;
   } else {
     socketRecvBuffer_->clearBuffer();
@@ -129,7 +199,7 @@ bool HttpServer::receiveBody()
   lastBody_.write(reinterpret_cast<const char*>(socketRecvBuffer_->getBuffer()),
                   length);
   socketRecvBuffer_->shiftBuffer(length);
-  return lastContentLength_ == static_cast<uint64_t>(lastBody_.tellp());
+  return lastContentLength_ == lastBody_.tellp();
 }
 
 std::string HttpServer::getBody() const
@@ -147,26 +217,35 @@ const std::string& HttpServer::getRequestPath() const
   return lastRequestHeader_->getRequestPath();
 }
 
-void HttpServer::feedResponse(const std::string& text, const std::string& contentType)
+void HttpServer::feedResponse(std::string& text, const std::string& contentType)
 {
-  feedResponse("200 OK", "", text, contentType);
+  feedResponse(200, "", text, contentType);
 }
 
-void HttpServer::feedResponse(const std::string& status,
+void HttpServer::feedResponse(int status,
                               const std::string& headers,
                               const std::string& text,
                               const std::string& contentType)
 {
   std::string httpDate = Time().toHTTPDate();
-  std::string header = "HTTP/1.1 ";
-  strappend(header, status, "\r\n",
-            "Date: ", httpDate, "\r\n",
-            "Content-Type: ", contentType, "\r\n");
-  strappend(header, "Content-Length: ", util::uitos(text.size()), "\r\n",
-            "Expires: ", httpDate, "\r\n",
-            "Cache-Control: no-cache\r\n");
+  std::string header= fmt("HTTP/1.1 %s\r\n"
+                          "Date: %s\r\n"
+                          "Content-Length: %lu\r\n"
+                          "Expires: %s\r\n"
+                          "Cache-Control: no-cache\r\n",
+                          getStatusString(status),
+                          httpDate.c_str(),
+                          static_cast<unsigned long>(text.size()),
+                          httpDate.c_str());
+  if(!contentType.empty()) {
+    header += "Content-Type: ";
+    header += contentType;
+    header += "\r\n";
+  }
   if(!allowOrigin_.empty()) {
-    strappend(header, "Access-Control-Allow-Origin: ", allowOrigin_, "\r\n");
+    header += "Access-Control-Allow-Origin: ";
+    header += allowOrigin_;
+    header += "\r\n";
   }
   if(supportsGZip()) {
     header += "Content-Encoding: gzip\r\n";
@@ -174,19 +253,27 @@ void HttpServer::feedResponse(const std::string& status,
   if(!supportsPersistentConnection()) {
     header += "Connection: close\r\n";
   }
-  if(!headers.empty()) {
-    header += headers;
-    if(!util::endsWith(headers, "\r\n")) {
-      header += "\r\n";
-    }
-  }
-
+  header += headers;
   header += "\r\n";
   A2_LOG_DEBUG(fmt("HTTP Server sends response:\n%s", header.c_str()));
   socketBuffer_.pushStr(header);
   socketBuffer_.pushStr(text);
 }
 
+void HttpServer::feedUpgradeResponse(const std::string& protocol,
+                                     const std::string& headers)
+{
+  std::string header= fmt("HTTP/1.1 101 Switching Protocols\r\n"
+                          "Upgrade: %s\r\n"
+                          "Connection: Upgrade\r\n"
+                          "%s"
+                          "\r\n",
+                          protocol.c_str(),
+                          headers.c_str());
+  A2_LOG_DEBUG(fmt("HTTP Server sends upgrade response:\n%s", header.c_str()));
+  socketBuffer_.pushStr(header);
+}
+
 ssize_t HttpServer::sendResponse()
 {
   return socketBuffer_.send();
@@ -203,19 +290,23 @@ bool HttpServer::authenticate()
     return true;
   }
 
-  std::string authHeader = lastRequestHeader_->getFirst("Authorization");
+  const std::string& authHeader =
+    lastRequestHeader_->find(HttpHeader::AUTHORIZATION);
   if(authHeader.empty()) {
     return false;
   }
-  std::pair<std::string, std::string> p;
-  util::divide(p, authHeader, ' ');
-  if(p.first != "Basic") {
+  std::pair<Scip, Scip> p;
+  util::divide(p, authHeader.begin(), authHeader.end(), ' ');
+  if(!util::streq(p.first.first, p.first.second, "Basic")) {
     return false;
   }
-  std::string userpass = Base64::decode(p.second);
-  std::pair<std::string, std::string> userpassPair;
-  util::divide(userpassPair, userpass, ':');
-  return username_ == userpassPair.first && password_ == userpassPair.second;
+  std::string userpass = base64::decode(p.second.first, p.second.second);
+  std::pair<Sip, Sip> up;
+  util::divide(up, userpass.begin(), userpass.end(), ':');
+  return util::streq(up.first.first, up.first.second,
+                     username_.begin(), username_.end()) &&
+    util::streq(up.second.first, up.second.second,
+                password_.begin(), password_.end());
 }
 
 void HttpServer::setUsernamePassword
