@@ -43,10 +43,11 @@
 #include <functional>
 
 #include "SharedHandle.h"
+#include "console.h"
 
 namespace aria2 {
 
-extern const std::string NO_DESCRIPTION;
+extern const char NO_DESCRIPTION[];
 extern const std::string NO_DEFAULT_VALUE;
 
 extern const std::string PATH_TO_FILE;
@@ -56,12 +57,12 @@ extern const std::string PATH_TO_DIR;
 extern const std::string PATH_TO_COMMAND;
 
 class Option;
+struct Pref;
 
 class OptionHandler {
 public:
   virtual ~OptionHandler() {}
   
-  virtual bool canHandle(const std::string& optName) = 0;
   virtual void parse(Option& option, const std::string& arg) = 0;
 
   virtual std::string createPossibleValuesString() const = 0;
@@ -72,9 +73,9 @@ public:
 
   virtual std::string toTagString() const = 0;
 
-  virtual const std::string& getName() const = 0;
+  virtual const char* getName() const = 0;
 
-  virtual const std::string& getDescription() const = 0;
+  virtual const char* getDescription() const = 0;
 
   virtual const std::string& getDefaultValue() const = 0;
 
@@ -82,6 +83,8 @@ public:
 
   virtual void hide() = 0;
 
+  virtual const Pref* getPref() const = 0;
+
   enum ARG_TYPE {
     REQ_ARG,
     OPT_ARG,
@@ -92,44 +95,28 @@ public:
 
   virtual char getShortName() const = 0;
 
-  virtual int getOptionID() const = 0;
-
-  virtual void setOptionID(int id) = 0;
-
   // Returns true if option value should be erased from argv to
   // prevent it from appearing in the output of ps.
   virtual bool getEraseAfterParse() const = 0;
 
   virtual void setEraseAfterParse(bool eraseAfterParse) = 0;
-};
 
-class OptionHandlerNameLesser:public std::binary_function
-<SharedHandle<OptionHandler>, SharedHandle<OptionHandler>, bool> {
-public:
-  bool operator()
-  (const SharedHandle<OptionHandler>& lhs,
-   const SharedHandle<OptionHandler>& rhs) const
-  {
-    return lhs->getName() < rhs->getName();
-  }
+  virtual bool getInitialOption() const = 0;
+  virtual void setInitialOption(bool f) = 0;
+  virtual bool getChangeOption() const = 0;
+  virtual void setChangeOption(bool f) = 0;
+  virtual bool getChangeOptionForReserved() const = 0;
+  virtual void setChangeOptionForReserved(bool f) = 0;
+  virtual bool getChangeGlobalOption() const = 0;
+  virtual void setChangeGlobalOption(bool f) = 0;
+  virtual bool getCumulative() const = 0;
+  virtual void setCumulative(bool f) = 0;
 };
 
-class OptionHandlerIDLesser:public std::binary_function
-<SharedHandle<OptionHandler>, SharedHandle<OptionHandler>, bool> {
-public:
-  bool operator()
-  (const SharedHandle<OptionHandler>& lhs,
-   const SharedHandle<OptionHandler>& rhs) const
-  {
-    return lhs->getOptionID() < rhs->getOptionID();
-  }
-};
-
-typedef SharedHandle<OptionHandler> OptionHandlerHandle;
-typedef std::vector<OptionHandlerHandle> OptionHandlers;
-
 std::ostream& operator<<(std::ostream& o, const OptionHandler& optionHandler);
 
+void write(const Console& out, const OptionHandler& optionHandler);
+
 } // namespace aria2
 
 #endif // D_OPTION_HANDLER_H
