@@ -9,9 +9,13 @@
 #include "Option.h"
 #include "FileEntry.h"
 #include "RarestPieceSelector.h"
-#include "InOrderPieceSelector.h"
+#include "InorderPieceSelector.h"
 #include "DownloadContext.h"
 #include "bittorrent_helper.h"
+#include "DiskAdaptor.h"
+#include "DiskWriterFactory.h"
+#include "PieceStatMan.h"
+#include "prefs.h"
 
 namespace aria2 {
 
@@ -20,7 +24,9 @@ class DefaultPieceStorageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST_SUITE(DefaultPieceStorageTest);
   CPPUNIT_TEST(testGetTotalLength);
   CPPUNIT_TEST(testGetMissingPiece);
+  CPPUNIT_TEST(testGetMissingPiece_many);
   CPPUNIT_TEST(testGetMissingPiece_excludedIndexes);
+  CPPUNIT_TEST(testGetMissingPiece_manyWithExcludedIndexes);
   CPPUNIT_TEST(testGetMissingFastPiece);
   CPPUNIT_TEST(testGetMissingFastPiece_excludedIndexes);
   CPPUNIT_TEST(testHasMissingPiece);
@@ -36,28 +42,25 @@ class DefaultPieceStorageTest:public CppUnit::TestFixture {
 private:
   SharedHandle<DownloadContext> dctx_;
   SharedHandle<Peer> peer;
-  Option* option;
+  SharedHandle<Option> option_;
   SharedHandle<PieceSelector> pieceSelector_;
 public:
   void setUp() {
+    option_.reset(new Option());
+    option_->put(PREF_DIR, ".");
     dctx_.reset(new DownloadContext());
-    bittorrent::load("test.torrent", dctx_);
+    bittorrent::load(A2_TEST_DIR"/test.torrent", dctx_, option_);
     peer.reset(new Peer("192.168.0.1", 6889));
     peer->allocateSessionResource(dctx_->getPieceLength(),
                                   dctx_->getTotalLength());
-    option = new Option();
-    pieceSelector_.reset(new InOrderPieceSelector());
-  }
-
-  void tearDown()
-  {
-    delete option;
-    option = 0;
+    pieceSelector_.reset(new InorderPieceSelector());
   }
 
   void testGetTotalLength();
   void testGetMissingPiece();
+  void testGetMissingPiece_many();
   void testGetMissingPiece_excludedIndexes();
+  void testGetMissingPiece_manyWithExcludedIndexes();
   void testGetMissingFastPiece();
   void testGetMissingFastPiece_excludedIndexes();
   void testHasMissingPiece();
@@ -75,34 +78,52 @@ public:
 CPPUNIT_TEST_SUITE_REGISTRATION(DefaultPieceStorageTest);
 
 void DefaultPieceStorageTest::testGetTotalLength() {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
 
-  CPPUNIT_ASSERT_EQUAL((uint64_t)384ULL, pss.getTotalLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)384LL, pss.getTotalLength());
 }
 
 void DefaultPieceStorageTest::testGetMissingPiece() {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   pss.setPieceSelector(pieceSelector_);
-  pss.setEndGamePieceNum(0);
-
   peer->setAllBitfield();
 
-  SharedHandle<Piece> piece = pss.getMissingPiece(peer);
+  SharedHandle<Piece> piece = pss.getMissingPiece(peer, 1);
   CPPUNIT_ASSERT_EQUAL(std::string("piece: index=0, length=128"),
                        piece->toString());
-  piece = pss.getMissingPiece(peer);
+  CPPUNIT_ASSERT(piece->usedBy(1));
+  piece = pss.getMissingPiece(peer, 1);
   CPPUNIT_ASSERT_EQUAL(std::string("piece: index=1, length=128"),
                        piece->toString());
-  piece = pss.getMissingPiece(peer);
+  piece = pss.getMissingPiece(peer, 1);
   CPPUNIT_ASSERT_EQUAL(std::string("piece: index=2, length=128"),
                        piece->toString());
-  piece = pss.getMissingPiece(peer);
-  CPPUNIT_ASSERT(piece.isNull());
+  piece = pss.getMissingPiece(peer, 1);
+  CPPUNIT_ASSERT(!piece);
+}
+
+void DefaultPieceStorageTest::testGetMissingPiece_many() {
+  DefaultPieceStorage pss(dctx_, option_.get());
+  pss.setPieceSelector(pieceSelector_);
+  peer->setAllBitfield();
+  std::vector<SharedHandle<Piece> > pieces;
+  pss.getMissingPiece(pieces, 2, peer, 1);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pieces.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=0, length=128"),
+                       pieces[0]->toString());
+  CPPUNIT_ASSERT(pieces[0]->usedBy(1));
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=1, length=128"),
+                       pieces[1]->toString());
+  pieces.clear();
+  pss.getMissingPiece(pieces, 2, peer, 1);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, pieces.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=2, length=128"),
+                       pieces[0]->toString());
 }
 
 void DefaultPieceStorageTest::testGetMissingPiece_excludedIndexes()
 {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   pss.setPieceSelector(pieceSelector_);
   pss.setEndGamePieceNum(0);
 
@@ -111,20 +132,38 @@ void DefaultPieceStorageTest::testGetMissingPiece_excludedIndexes()
   std::vector<size_t> excludedIndexes;
   excludedIndexes.push_back(1);
 
-  SharedHandle<Piece> piece = pss.getMissingPiece(peer, excludedIndexes);
+  SharedHandle<Piece> piece = pss.getMissingPiece(peer, excludedIndexes, 1);
   CPPUNIT_ASSERT_EQUAL(std::string("piece: index=0, length=128"),
                        piece->toString());
 
-  piece = pss.getMissingPiece(peer, excludedIndexes);
+  piece = pss.getMissingPiece(peer, excludedIndexes, 1);
   CPPUNIT_ASSERT_EQUAL(std::string("piece: index=2, length=128"),
                        piece->toString());
 
-  piece = pss.getMissingPiece(peer, excludedIndexes);
-  CPPUNIT_ASSERT(piece.isNull());
+  piece = pss.getMissingPiece(peer, excludedIndexes, 1);
+  CPPUNIT_ASSERT(!piece);
+}
+
+void DefaultPieceStorageTest::testGetMissingPiece_manyWithExcludedIndexes() {
+  DefaultPieceStorage pss(dctx_, option_.get());
+  pss.setPieceSelector(pieceSelector_);
+  peer->setAllBitfield();
+  std::vector<size_t> excludedIndexes;
+  excludedIndexes.push_back(1);
+  std::vector<SharedHandle<Piece> > pieces;
+  pss.getMissingPiece(pieces, 2, peer, excludedIndexes, 1);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pieces.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=0, length=128"),
+                       pieces[0]->toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=2, length=128"),
+                       pieces[1]->toString());
+  pieces.clear();
+  pss.getMissingPiece(pieces, 2, peer, excludedIndexes, 1);
+  CPPUNIT_ASSERT(pieces.empty());
 }
 
 void DefaultPieceStorageTest::testGetMissingFastPiece() {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   pss.setPieceSelector(pieceSelector_);
   pss.setEndGamePieceNum(0);
 
@@ -132,16 +171,16 @@ void DefaultPieceStorageTest::testGetMissingFastPiece() {
   peer->setFastExtensionEnabled(true);
   peer->addPeerAllowedIndex(2);
 
-  SharedHandle<Piece> piece = pss.getMissingFastPiece(peer);
+  SharedHandle<Piece> piece = pss.getMissingFastPiece(peer, 1);
   CPPUNIT_ASSERT_EQUAL(std::string("piece: index=2, length=128"),
                        piece->toString());
 
-  CPPUNIT_ASSERT(pss.getMissingFastPiece(peer).isNull());
+  CPPUNIT_ASSERT(!pss.getMissingFastPiece(peer, 1));
 }
 
 void DefaultPieceStorageTest::testGetMissingFastPiece_excludedIndexes()
 {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   pss.setPieceSelector(pieceSelector_);
   pss.setEndGamePieceNum(0);
 
@@ -153,15 +192,15 @@ void DefaultPieceStorageTest::testGetMissingFastPiece_excludedIndexes()
   std::vector<size_t> excludedIndexes;
   excludedIndexes.push_back(2);
 
-  SharedHandle<Piece> piece = pss.getMissingFastPiece(peer, excludedIndexes);
+  SharedHandle<Piece> piece = pss.getMissingFastPiece(peer, excludedIndexes, 1);
   CPPUNIT_ASSERT_EQUAL(std::string("piece: index=1, length=128"),
                        piece->toString());
   
-  CPPUNIT_ASSERT(pss.getMissingFastPiece(peer, excludedIndexes).isNull());
+  CPPUNIT_ASSERT(!pss.getMissingFastPiece(peer, excludedIndexes, 1));
 }
 
 void DefaultPieceStorageTest::testHasMissingPiece() {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
 
   CPPUNIT_ASSERT(!pss.hasMissingPiece(peer));
   
@@ -171,61 +210,61 @@ void DefaultPieceStorageTest::testHasMissingPiece() {
 }
 
 void DefaultPieceStorageTest::testCompletePiece() {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   pss.setPieceSelector(pieceSelector_);
   pss.setEndGamePieceNum(0);
 
   peer->setAllBitfield();
 
-  SharedHandle<Piece> piece = pss.getMissingPiece(peer);
+  SharedHandle<Piece> piece = pss.getMissingPiece(peer, 1);
   CPPUNIT_ASSERT_EQUAL(std::string("piece: index=0, length=128"),
                        piece->toString());
 
-  CPPUNIT_ASSERT_EQUAL((uint64_t)0ULL, pss.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, pss.getCompletedLength());
 
   pss.completePiece(piece);
 
-  CPPUNIT_ASSERT_EQUAL((uint64_t)128ULL, pss.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)128LL, pss.getCompletedLength());
 
-  SharedHandle<Piece> incompletePiece = pss.getMissingPiece(peer);
+  SharedHandle<Piece> incompletePiece = pss.getMissingPiece(peer, 1);
   incompletePiece->completeBlock(0);
-  CPPUNIT_ASSERT_EQUAL((uint64_t)256ULL, pss.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)256LL, pss.getCompletedLength());
 }
 
 void DefaultPieceStorageTest::testGetPiece() {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   
   SharedHandle<Piece> pieceGot = pss.getPiece(0);
   CPPUNIT_ASSERT_EQUAL((size_t)0, pieceGot->getIndex());
-  CPPUNIT_ASSERT_EQUAL((size_t)128, pieceGot->getLength());
+  CPPUNIT_ASSERT_EQUAL(128, pieceGot->getLength());
   CPPUNIT_ASSERT_EQUAL(false, pieceGot->pieceComplete());
 }
 
 void DefaultPieceStorageTest::testGetPieceInUsedPieces() {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   SharedHandle<Piece> piece = SharedHandle<Piece>(new Piece(0, 128));
   piece->completeBlock(0);
   pss.addUsedPiece(piece);
   SharedHandle<Piece> pieceGot = pss.getPiece(0);
   CPPUNIT_ASSERT_EQUAL((size_t)0, pieceGot->getIndex());
-  CPPUNIT_ASSERT_EQUAL((size_t)128, pieceGot->getLength());
+  CPPUNIT_ASSERT_EQUAL(128, pieceGot->getLength());
   CPPUNIT_ASSERT_EQUAL((size_t)1, pieceGot->countCompleteBlock());
 }
 
 void DefaultPieceStorageTest::testGetPieceCompletedPiece() {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   SharedHandle<Piece> piece = SharedHandle<Piece>(new Piece(0, 128));
   pss.completePiece(piece);
   SharedHandle<Piece> pieceGot = pss.getPiece(0);
   CPPUNIT_ASSERT_EQUAL((size_t)0, pieceGot->getIndex());
-  CPPUNIT_ASSERT_EQUAL((size_t)128, pieceGot->getLength());
+  CPPUNIT_ASSERT_EQUAL(128, pieceGot->getLength());
   CPPUNIT_ASSERT_EQUAL(true, pieceGot->pieceComplete());
 }
 
 void DefaultPieceStorageTest::testCancelPiece()
 {
   size_t pieceLength = 256*1024;
-  uint64_t totalLength = 32*pieceLength; // <-- make the number of piece greater than END_GAME_PIECE_NUM
+  off_t totalLength = 32*pieceLength; // <-- make the number of piece greater than END_GAME_PIECE_NUM
   std::deque<std::string> uris1;
   uris1.push_back("http://localhost/src/file1.txt");
   SharedHandle<FileEntry> file1(new FileEntry("src/file1.txt", totalLength, 0 /*, uris1*/));
@@ -233,26 +272,28 @@ void DefaultPieceStorageTest::testCancelPiece()
   SharedHandle<DownloadContext> dctx
     (new DownloadContext(pieceLength, totalLength, "src/file1.txt"));
 
-  SharedHandle<DefaultPieceStorage> ps(new DefaultPieceStorage(dctx, option));
+  SharedHandle<DefaultPieceStorage> ps(new DefaultPieceStorage(dctx, option_.get()));
 
-  SharedHandle<Piece> p = ps->getMissingPiece(0);
+  SharedHandle<Piece> p = ps->getMissingPiece(0, 1);
   p->completeBlock(0);
   
-  ps->cancelPiece(p);
+  ps->cancelPiece(p, 1);
 
-  SharedHandle<Piece> p2 = ps->getMissingPiece(0);
+  SharedHandle<Piece> p2 = ps->getMissingPiece(0, 2);
 
   CPPUNIT_ASSERT(p2->hasBlock(0));
+  CPPUNIT_ASSERT(p2->usedBy(2));
+  CPPUNIT_ASSERT(!p2->usedBy(1));
 }
 
 void DefaultPieceStorageTest::testMarkPiecesDone()
 {
   size_t pieceLength = 256*1024;
-  uint64_t totalLength = 4*1024*1024;
+  off_t totalLength = 4*1024*1024;
   SharedHandle<DownloadContext> dctx
     (new DownloadContext(pieceLength, totalLength));
 
-  DefaultPieceStorage ps(dctx, option);
+  DefaultPieceStorage ps(dctx, option_.get());
 
   ps.markPiecesDone(pieceLength*10+16*1024*2+1);
 
@@ -262,7 +303,7 @@ void DefaultPieceStorageTest::testMarkPiecesDone()
   for(size_t i = 10; i < (totalLength+pieceLength-1)/pieceLength; ++i) {
     CPPUNIT_ASSERT(!ps.hasPiece(i));
   }
-  CPPUNIT_ASSERT_EQUAL((uint64_t)pieceLength*10+16*1024*2, ps.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)pieceLength*10+16*1024*2, ps.getCompletedLength());
 
   ps.markPiecesDone(totalLength);
 
@@ -271,7 +312,7 @@ void DefaultPieceStorageTest::testMarkPiecesDone()
   }
 
   ps.markPiecesDone(0);
-  CPPUNIT_ASSERT_EQUAL((uint64_t)0, ps.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)0, ps.getCompletedLength());
 }
 
 void DefaultPieceStorageTest::testGetCompletedLength()
@@ -279,12 +320,12 @@ void DefaultPieceStorageTest::testGetCompletedLength()
   SharedHandle<DownloadContext> dctx
     (new DownloadContext(1024*1024, 256*1024*1024));
   
-  DefaultPieceStorage ps(dctx, option);
+  DefaultPieceStorage ps(dctx, option_.get());
   
-  CPPUNIT_ASSERT_EQUAL((uint64_t)0, ps.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)0, ps.getCompletedLength());
 
   ps.markPiecesDone(250*1024*1024);
-  CPPUNIT_ASSERT_EQUAL((uint64_t)250*1024*1024, ps.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)250*1024*1024, ps.getCompletedLength());
 
   std::vector<SharedHandle<Piece> > inFlightPieces;
   for(int i = 0; i < 2; ++i) {
@@ -293,26 +334,26 @@ void DefaultPieceStorageTest::testGetCompletedLength()
       p->completeBlock(j);
     }
     inFlightPieces.push_back(p);
-    CPPUNIT_ASSERT_EQUAL((size_t)512*1024, p->getCompletedLength());
+    CPPUNIT_ASSERT_EQUAL(512*1024, p->getCompletedLength());
   }
   ps.addInFlightPiece(inFlightPieces);
   
-  CPPUNIT_ASSERT_EQUAL((uint64_t)251*1024*1024, ps.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)251*1024*1024, ps.getCompletedLength());
 
   ps.markPiecesDone(256*1024*1024);
 
-  CPPUNIT_ASSERT_EQUAL((uint64_t)256*1024*1024, ps.getCompletedLength());
+  CPPUNIT_ASSERT_EQUAL((off_t)256*1024*1024, ps.getCompletedLength());
 }
 
 void DefaultPieceStorageTest::testGetNextUsedIndex()
 {
-  DefaultPieceStorage pss(dctx_, option);
+  DefaultPieceStorage pss(dctx_, option_.get());
   CPPUNIT_ASSERT_EQUAL((size_t)3, pss.getNextUsedIndex(0));
-  SharedHandle<Piece> piece = pss.getMissingPiece(2);
+  SharedHandle<Piece> piece = pss.getMissingPiece(2, 1);
   CPPUNIT_ASSERT_EQUAL((size_t)2, pss.getNextUsedIndex(0));
   pss.completePiece(piece);
   CPPUNIT_ASSERT_EQUAL((size_t)2, pss.getNextUsedIndex(0));
-  piece = pss.getMissingPiece(0);
+  piece = pss.getMissingPiece(0, 1);
   CPPUNIT_ASSERT_EQUAL((size_t)2, pss.getNextUsedIndex(0));
 }
 
