@@ -1,7 +1,10 @@
-#ifndef _D_FIXED_NUMBER_RANDOMIZER_H_
-#define _D_FIXED_NUMBER_RANDOMIZER_H_
+#ifndef D_FIXED_NUMBER_RANDOMIZER_H
+#define D_FIXED_NUMBER_RANDOMIZER_H
 
 #include "Randomizer.h"
+#include <cstdlib>
+
+namespace aria2 {
 
 class FixedNumberRandomizer : public Randomizer {
 private:
@@ -33,4 +36,6 @@ public:
   }
 };
 
-#endif // _D_FIXED_NUMBER_RANDOMIZER_H_
+} // namespace aria2
+
+#endif // D_FIXED_NUMBER_RANDOMIZER_H
