@@ -15,6 +15,7 @@
 #include "File.h"
 #include "array_fun.h"
 #include "RecoverableException.h"
+#include "util.h"
 
 namespace aria2 {
 
@@ -48,8 +49,8 @@ CPPUNIT_TEST_SUITE_REGISTRATION( RequestGroupManTest );
 
 void RequestGroupManTest::testIsSameFileBeingDownloaded()
 {
-  SharedHandle<RequestGroup> rg1(new RequestGroup(option_));
-  SharedHandle<RequestGroup> rg2(new RequestGroup(option_));
+  SharedHandle<RequestGroup> rg1(new RequestGroup(util::copy(option_)));
+  SharedHandle<RequestGroup> rg2(new RequestGroup(util::copy(option_)));
 
   SharedHandle<DownloadContext> dctx1
     (new DownloadContext(0, 0, "aria2.tar.bz2"));
@@ -84,7 +85,7 @@ void RequestGroupManTest::testSaveServerStat()
     (std::vector<SharedHandle<RequestGroup> >(),0,option_.get());
   SharedHandle<ServerStat> ss_localhost(new ServerStat("localhost", "http"));
   rm.addServerStat(ss_localhost);
-  File f("./aria2_RequestGroupManTest_testSaveServerStat");
+  File f(A2_TEST_OUT_DIR"/aria2_RequestGroupManTest_testSaveServerStat");
   if(f.exists()) {
     f.remove();
   }
@@ -98,7 +99,7 @@ void RequestGroupManTest::testSaveServerStat()
 
 void RequestGroupManTest::testLoadServerStat()
 {
-  File f("./aria2_RequestGroupManTest_testLoadServerStat");
+  File f(A2_TEST_OUT_DIR"/aria2_RequestGroupManTest_testLoadServerStat");
   std::ofstream o(f.getPath().c_str(), std::ios::binary);
   o << "host=localhost, protocol=http, dl_speed=0, last_updated=1219505257,"
     << "status=OK";
@@ -110,17 +111,17 @@ void RequestGroupManTest::testLoadServerStat()
   CPPUNIT_ASSERT(rm.loadServerStat(f.getPath()));
   SharedHandle<ServerStat> ss_localhost = rm.findServerStat("localhost",
                                                             "http");
-  CPPUNIT_ASSERT(!ss_localhost.isNull());
+  CPPUNIT_ASSERT(ss_localhost);
   CPPUNIT_ASSERT_EQUAL(std::string("localhost"), ss_localhost->getHostname());
 }
 
 void RequestGroupManTest::testChangeReservedGroupPosition()
 {
   SharedHandle<RequestGroup> gs[] = {
-    SharedHandle<RequestGroup>(new RequestGroup(option_)),
-    SharedHandle<RequestGroup>(new RequestGroup(option_)),
-    SharedHandle<RequestGroup>(new RequestGroup(option_)),
-    SharedHandle<RequestGroup>(new RequestGroup(option_))
+    SharedHandle<RequestGroup>(new RequestGroup(util::copy(option_))),
+    SharedHandle<RequestGroup>(new RequestGroup(util::copy(option_))),
+    SharedHandle<RequestGroup>(new RequestGroup(util::copy(option_))),
+    SharedHandle<RequestGroup>(new RequestGroup(util::copy(option_)))
   };
   std::vector<SharedHandle<RequestGroup> > groups(vbegin(gs), vend(gs));
   RequestGroupMan rm(groups, 0, option_.get());
